///////////////////////////////////////////////////////////////////////////////
//
//  Filename: CoordTrans.cpp
//  Author: Wang, Yi
//  Last modified: 2004-1-26 11:53
//
///////////////////////////////////////////////////////////////////////////////

#include "CoordTrans.h"

CoordTrans::CoordTrans(double width, double height)
{
	wd = width;
	ht = height;
	sc = 1; x0 = 0; y0 = 0;
}

void CoordTrans::SetViewSize(double width, double height)
{
	wd = width;
	ht = height;
}

void CoordTrans::Map(double x1, double y1, double &x2, double &y2)
{
	x2 = sc * x1 + x0;
	y2 = sc * y1 + y0;
}

void CoordTrans::Map(tagVect &v1, tagVect &v2)
{
	Map(v1.x, v1.y, v2.x, v2.y);
}

void CoordTrans::Map(tagCircle &c1, tagCircle &c2)
{
	Map(c1.ox, c1.oy, c2.ox, c2.oy);
	c2.r = c1.r * sc;
}

void CoordTrans::Map(tagArc &arc1, tagArc &arc2)
{
	arc2 = arc1;
	Map(arc1.c, arc2.c);
}

void CoordTrans::MapRev(double x1, double y1, double &x2, double &y2)
{
	x2=(x1-x0) / sc;
	y2=(y1-y0) / sc;
}

void CoordTrans::MapRev(tagVect &v1, tagVect &v2)
{
	MapRev(v1.x, v1.y, v2.x, v2.y);
}

double CoordTrans::Scale(double rate)
{
	return Scale(rate, wd / 2, ht / 2);
}

double CoordTrans::Scale(double rate, double xc, double yc)
{
	if (rate>0) 
	{
		sc *= rate;
		x0 = (1-rate) * xc + rate * x0;
		y0 = (1-rate) * yc + rate * y0;
	}
	
	return sc;
}

void CoordTrans::Move(double dx, double dy)
{
	x0 += dx; y0 += dy;
}

void CoordTrans::MoveTo(double x, double y)
{
	x0 = x; y0 = y;
}
