///////////////////////////////////////////////////////////////////////////////
//
//  Filename: Track.cpp
//  Author: Wang, Yi
//  Last modified: 2004-2-25 3:53
//
///////////////////////////////////////////////////////////////////////////////

#include "Track.h"


/*----------------------------------------------------------------------------*/
/*
 * Description: Constructor
 * Para: none
 * Return: none
 */

Track::Track(void)
{
	AchseLen = 20;
	StepLen = 1;
	Grid = 10;
    Omega = StepLen / AchseLen;
    ResetPos.time  = 0;
	ResetPos.speed = 0;
	ResetPos.vect.alpha = 0;
	ResetPos.vect.x = 0;
	ResetPos.vect.y = 0;
    Reset();
}

/*----------------------------------------------------------------------------*/
/*
 * Description: move a step
 * Para<l>: left step, 0 ,1 -1
 * Para<r>: right step, 0 ,1 -1
 * Return: none
 */
void Track::AppendCmd(QString &cmd)
{
	if (cmd.left(2)!="$G")
	{
		return;
	}
	
	bool IsOk;
	int time = cmd.mid(2, 4).toUShort(&IsOk, 16);
	int c = (cmd.mid(7, 1)=="V") ? 1 : -1;

	if (cmd.mid(6, 1)=="L")
	{
		Step(time, c, 0);
	}
	else
	{
		Step(time, 0, c);
	}
}

/*----------------------------------------------------------------------------*/
/*
 * Description: move a step
 * Para<l>: left step, 0 ,1 -1
 * Para<r>: right step, 0 ,1 -1
 * Return: none
 */
void Track::Step(int time ,int l, int r)
{
    int step, lk1;
    double alpha1;

    step = l + r;
    lk1 = l - r;
    CurPos.time = time;
    alpha1 = CurPos.vect.alpha;
    CurPos.vect.alpha += lk1 * Omega;

    if (lk0==1){
        if (lk1==-1){
            alpha1 -= Omega;
            lk0 = 0;
        }
    }
    else if (lk0==-1){
        if (lk1==1){
            alpha1 += Omega;
            lk0 = 0;
        }
    }
    else{
        lk0 = lk1;
    }

    CurPos.vect.x += (step * StepLen / 2 * cos(alpha1));
    CurPos.vect.y += (step * StepLen / 2 * sin(alpha1));
    
	double ds = sqrt( pow((CurPos.vect.x - OldPos.vect.x), 2) 
    			+ pow((CurPos.vect.y - OldPos.vect.y), 2));

	CurPos.speed = ds / ( CurPos.time - OldPos.time );

    if ( (ds>=Grid) || (vec.count()==0))
    {
    	vec.append(CurPos);
		OldPos = CurPos;
    }
}

/*----------------------------------------------------------------------------*/
/*
 * Description: access to an item
 * Para<index>: index of the item
 * Return: reference of the item
 */
tagPos &Track::Item(int index)
{
	return vec[index];
}

/*----------------------------------------------------------------------------*/
/*
 * Description: reset the list
 * Para: none
 * Return: none
 */
void Track::Reset(void)
{
	vec.clear();

    lk0=0;
    CurPos = ResetPos;
    OldPos = ResetPos;
    vec.append(ResetPos);
}

/*----------------------------------------------------------------------------*/
/*
 * Description: reset the list
 * Para<pos>: ResetPos
 * Return: none
 */
void Track::Reset(tagPos & pos)
{
	ResetPos = pos;
	Reset();
}

/*----------------------------------------------------------------------------*/
/*
 * Description: length of the list
 * Para: none
 * Return: count of listitems
 */
int Track::GetCount(void)
{
	return vec.count();
}