///////////////////////////////////////////////////////////////////////////////
//
//  Filename: form1.cpp
//  Author: Wang, Yi
//  Last modified: 2004-2-25 3:57
//
///////////////////////////////////////////////////////////////////////////////

#include "form1.h"

/***********************  Global  *********************************************/
const double PointGross = 3;
const double ArrowLen = 30;  // the length of Arrow 
const double SearchR = 6;

BahnDesign *pBD;
Track *pTrack;
BTool CurTool;
Form1 *pForm1;
CoordTrans *ct;

QString BTStr[] = {"Normal",				"NormalId",				"Add", 				"Delete",
				   "DeleteId",				"Move", 				"MoveId", 			"Moving",
				   "RotateId",				"Rotating",				"VMove1", 			"VMove2"};

int BTCursor[]=	{Qt::ArrowCursor,		Qt::PointingHandCursor,	Qt::CrossCursor,	Qt::ArrowCursor,
				 Qt::PointingHandCursor,	Qt::ArrowCursor,		Qt::SizeAllCursor,	Qt::SizeAllCursor,
				 Qt::SizeVerCursor,		Qt::SizeVerCursor,		Qt::SizeAllCursor,	Qt::SizeAllCursor};

double OldX, OldY;
double CurX, CurY;
int SelBDId = -1;  // selected BD point
int onBDId = -1; // mouse point on it or its arrow

/*----------------------  Colors  --------------------------------------------*/
const QColor clrBDSel = Qt::yellow;  // Color of selected BahnDesign Point 
const QColor clrVBG = Qt::white;  // Color of ViewerBackground
const QColor clrBDNorm = Qt::blue;  // Color of normal BahnDesign Point 


/***********************  Member of ViewerWidget ******************************/

ViewerWidget::ViewerWidget(QWidget *parent, const char *name):QWidget(parent, name)
{
	QWidget(parent, name);
	ct = new CoordTrans(width(), height());
};

/*----------------------------------------------------------------------------*/
void ViewerWidget::drawBahnStrecken()
{
	double pr = PointGross;
	pPainter->setPen(Qt::black);
	
	for (int i = 0;i<pBD->StkCt;i++) 
	{
		tagBahnStrecke &stk = pBD->Stks[i];
		tagVect &vect = stk.VectSt;
		tagVect &vect1 = stk.VectEd;
		double x, y, x1, y1;
		ct->Map(vect.x, vect.y, x, y);
		ct->Map(vect1.x, vect1.y, x1, y1);

		if (stk.Type == 0) 
		{ // line
			pPainter->drawLine(x, y, x1, y1);
		}
		else 
		{ // kreise
			tagArc &arc = stk.arc;
			tagCircle c1;
			ct->Map(arc.c, c1);
			double alen = arc.alen / pi * 180.0 * 16.0;
			pPainter->drawArc(c1.ox - c1.r, c1.oy - c1.r, 2.0 * c1.r, 2.0 * c1.r, 
				(1 - stk.arc.a / (2.0 * pi)) * 360.0 * 16.0, alen);
		}

		pPainter->drawRect(x - pr, y - pr, pr * 2, pr * 2);
	}
}

/*----------------------  Draw a Vector on Viewer  ---------------------------*/
void ViewerWidget::_drawVect(int id, tagVect &vect, QColor &tColor)
{
	double pr = PointGross, vl = ArrowLen;
	double x, y;
	
	pPainter->setPen(tColor);
	ct->Map(vect.x, vect.y, x, y);
	pPainter->drawLine(x, y, x + vl * cos(vect.alpha), y + vl * sin(vect.alpha));
	pPainter->drawRect(x - pr, y - pr, pr * 2, pr * 2);
	pPainter->drawText(x + 10, y + 15, QString::number(id));
}

/*----------------------  Select a BahnDesignPoint  --------------------------*/
void ViewerWidget::SelBahnDesignPt(int id)
{
	int id0 = SelBDId;
	SelBDId = id;
	drawBahnDesignPt(id0, true);
	drawBahnDesignPt(id, true);
	pForm1->ShowBDPtData(id);
}

/*----------------------  Move a BahnDesignPoint  ----------------------------*/
void ViewerWidget::moveBahnDesignPt(double tx, double ty)
{
	int id = onBDId;
	drawBahnDesignPt(id, false);
	pBD->Pts[id].Vect.x = tx;
	pBD->Pts[id].Vect.y = ty;
	drawBahnDesignPt(id, true);
	
	if (id==SelBDId)
	{
		pForm1->ShowBDPtData(id);
	}
	
	pBD->Calc();
	repaint();
	pForm1->SpeedViewer->repaint();
}

/*----------------------  rotate a BahnDesignPoint  --------------------------*/
void ViewerWidget::rotateBahnDesignPt(double tx, double ty)
{
	int id = onBDId;
	drawBahnDesignPt(id, false);
	tagVect *pVect = &(pBD->Pts[id].Vect);
	double dy = ty - pVect->y;
	double dx = tx - pVect->x;
	pVect->alpha = atan2(dy, dx);
	drawBahnDesignPt(id, true);
	
	if (id==SelBDId) 
	{
		pForm1->ShowBDPtData(id);
	}
	
	pBD->Calc();
	repaint();
	pForm1->SpeedViewer->repaint();
}

/*----------------------  Draw a Bahn DesignPoint  ---------------------------*/
void ViewerWidget::drawBahnDesignPt(int id, bool IsNotClear)
{
	if (id>=0) 
	{
		tagVect &vect = pBD->Pts[id].Vect;
		QColor tColor;
		
		if (IsNotClear)
		{
			tColor = (id==SelBDId) ? clrBDSel : clrBDNorm;
		}
		else
		{
			tColor = clrVBG;	
		}
		
		_drawVect(id, vect, tColor);
	}
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::drawBahnDesign()
{
	int i;
	
	for (i=0; i<pBD->Count; i++) 
	{
		drawBahnDesignPt(i, true);
	}
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::drawTrack()
{
	int i, Ct = pTrack->GetCount();
	double x1, x2, y1, y2;

	if (Ct<2)
	{
		return;
	}

	pPainter->setPen(Qt::red);
	tagPos &pos1 = pTrack->Item(0);
	ct->Map(pos1.vect.x, pos1.vect.y, x1, y1);

	for (i=1; i<Ct-1; i++) 
	{
		tagPos &pos2 = pTrack->Item(i);
		ct->Map(pos2.vect.x, pos2.vect.y, x2, y2);
		pPainter->drawLine(x1, y1, x2, y2);
		x1 = x2; y1 = y2;			
	}
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::resizeEvent( QResizeEvent *e)
{
	QSize s = e->size();
	ct->SetViewSize(s.width(), s.height());
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::paintEvent( QPaintEvent * e)
{
	pPainter = new QPainter(this);

  tagCalcGeoIContext &gc = pBD->gc;
	if (pBD->Count>1) 
	{
		pPainter->setPen(Qt::green);
		
		for (int i=0;i<4;i++) 
		{
			if (i==2)
			{
				pPainter->setPen(Qt::cyan);
			}
			
			tagCircle &c0 = gc.c[i], c;
			ct->Map(c0,c);
			pPainter->drawEllipse( c.ox - c.r, c.oy - c.r, 2 * c.r, 2 * c.r );
			pPainter->drawText( c.ox + 10, c.oy + 10, QString::number(i) );
		}
	}

	drawBahnStrecken();
	drawTrack();
	drawBahnDesign();
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::wheelEvent ( QWheelEvent *e )
{
	int delta = e->delta();
	double x = e->x();
	double y = e->y();

	if (delta>0)
	{
		ct->Scale(2, x, y);
	}
	else
	{
		ct->Scale(0.5, x, y);
	}
	repaint();
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::mousePressEvent( QMouseEvent *e)
{
	double x = e->x(), y = e->y();
	
	if (e->button()==Qt::LeftButton) 
	{ //left
		switch (CurTool) 
		{
			case BT_Add: 
			{
				tagBahnDesignPt *pNewPt = pBD->add();
				ct->MapRev(x, y, pNewPt->Vect.x, pNewPt->Vect.y);
				pBD->Calc();
				repaint();
				pForm1->SpeedViewer->repaint();
			}
			break;
			case V_Move1: 
			{
				pForm1->SetCurTool(V_Move2);
				OldX = x; OldY = y;
			}
			break;
			case BT_NormalId: 
			{
				SelBahnDesignPt( onBDId );
				pForm1->SetCurTool(BT_Normal);
			}
			break;
			case BT_MoveId: 
			{
				pForm1->SetCurTool(BT_Moving);
			}
			break;
			case BT_DeleteId: 
			{
				pBD->remove(onBDId);
				pBD->Calc();
				repaint();
				pForm1->SpeedViewer->repaint();
			}
			break;
			case BT_RotateId: 
			{
				pForm1->SetCurTool(BT_Rotating);
			}
			break;
		}
	}
	else{ // right
		pForm1->SetCurTool(BT_Normal);
		SelBahnDesignPt(-1);
	}	
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::mouseMoveEvent( QMouseEvent * e )
{
	double x, y;
	x = e->x();

	y = e->y();
	ct->MapRev(x, y, CurX, CurY);
	ButtonState btn = e->state();

	if (btn==Qt::LeftButton) 
	{  // mousedrag
		switch (CurTool) 
		{
			case V_Move2: 
			{ // move Viewer
				ct->Move( x - OldX, y - OldY );
				OldX = x; OldY = y;
				repaint();
			}
			break;
			case BT_Moving: 
			{ // moving a BahnDesign point
				moveBahnDesignPt( CurX, CurY );
			}
			break;
			case BT_Rotating: 
			{ // rotating a BahnDesign point
				rotateBahnDesignPt( CurX, CurY );
			}
			break;
		}
	}
	else if (btn==Qt::NoButton) 
	{ // mousemove
		onBDId=pBD->SearchBDPt( CurX, CurY, SearchR / ct->sc, 0, false );

		switch (CurTool) 
		{
			case BT_Move: 
			{
				if ( onBDId >= 0 ) 
				{ // a BahnDesign point is found
					pForm1->SetCurTool(BT_MoveId);
				} 
				else 
				{
					onBDId = pBD->SearchBDPt( CurX, CurY, SearchR / ct->sc, 
							ArrowLen / ct->sc, true );
					if ( onBDId >= 0 ) 
					{ // an arrow is found
						pForm1->SetCurTool(BT_RotateId);
					}
				}
			}
			break;

			case BT_MoveId: 
			{
				if ( onBDId < 0 ) 
				{
					pForm1->SetCurTool( BT_Move );
				}
			}
			break;
			case BT_RotateId: 
			{
				onBDId = pBD->SearchBDPt( CurX, CurY, SearchR / ct->sc, 
					ArrowLen / ct->sc, true );
				if (onBDId<0) 
				{
					pForm1->SetCurTool( BT_Move );
				}
			}
			break;
			case BT_Normal: 
			{
				if ( onBDId >= 0 )
				{
					pForm1->SetCurTool( BT_NormalId );
				}
			}
			break;
			case BT_NormalId: 
			{
				if ( onBDId < 0 )
				{
					pForm1->SetCurTool( BT_Normal );
				}
			}
			break;
			case BT_Delete: 
			{
				if ( onBDId >= 0 )
				{
					pForm1->SetCurTool( BT_DeleteId );
				}
			}
			break;
			case BT_DeleteId: 
			{
				if ( onBDId < 0 )
				{
					pForm1->SetCurTool( BT_Delete );
				}
			}
			break;
		}
	}
	
	pForm1->ShowCurStatus();
}

/*----------------------------------------------------------------------------*/
void ViewerWidget::mouseReleaseEvent( QMouseEvent * e )
{
	if ( e->button() == Qt::LeftButton ) 
	{ //left
		switch ( CurTool ) 
		{
			case V_Move2: 
			{
				pForm1->SetCurTool(V_Move1);
			}
			break;
			case BT_Moving: 
			{
				pForm1->SetCurTool(BT_MoveId);
			}
			break;
			case BT_Rotating: 
			{
				pForm1->SetCurTool(BT_RotateId);
			}
			break;
		}
	}
}



/***********************  Member of SpeedWidget ******************************/

SpeedWidget::SpeedWidget(QWidget *parent, const char *name):QWidget(parent, name)
{
	QWidget(parent, name);
};

/*----------------------------------------------------------------------------*/
void SpeedWidget::paintEvent( QPaintEvent * e)
{
	QPainter *pnt = new QPainter(this);

	int i, Ct, wd = width(), h0 = 110, hmax = 10;
	
	// MaxSpeed
	pnt->setPen(Qt::red);
	pnt->drawLine( 0, hmax, wd, hmax );

	// 0
	pnt->setPen(Qt::black);
	pnt->drawLine( 0, h0, wd, h0 );

	int x1, y1, x2, y2;

	// soll
	Ct = pBD->SpdList.GetCount();
	if ( Ct > 1 )
	{
		pnt->setPen(Qt::blue);
		tagSpeedListItem *pItem;;

		for (i=0;i<Ct;i++)
		{
			pItem = &(pBD->SpdList.Item(i));
			x1 = pItem->t0;
			x2 = x1 + pItem->dt;
			y1 = 110 - pItem->v0;
			y2 = 110 - (pItem->v0 + pItem->a * pItem->dt);
			pnt->drawLine( x1, y1, x2, y2 );
		}
	}


	// ist
	Ct = pTrack->GetCount();
	if ( Ct > 1 )
	{
		pnt->setPen(Qt::green);
		tagPos & pos1 = pTrack->Item(0);
		x1 = pos1.time;
		y1 = 110 - pos1.speed;

		for (i=1;i<Ct;i++)
		{
			tagPos & pos2 = pTrack->Item(i);
			x2 = pos2.time;
			y2 = 110 - pos2.speed;
			pnt->drawLine( x1, y1, x2, y2 );
			x1 = x2; y1 = y2;
		}
	}
}

/***********************  Member of Form1 *************************************/

/*----------------------  Show current status on the statusbar  --------------*/
void Form1::ShowCurStatus()
{
	QString msg = "[" + BTStr[CurTool] + "]";
	msg += "\t(" + QString::number(CurX) + "," + QString::number(CurY) + ")";
	statusBar()->message( msg );
}

/*----------------------  Show Data in PlanSheet  ----------------------------*/
void Form1::ShowBDPtData(int id)
{
	if (id<0) 
	{
		tabPlan->setDisabled(true);
		leId->setText("");
		leTime->setText("");
		leSpeed->setText("");
		leX->setText("");
		leY->setText("");
		leAlpha->setText("");
	}
	else 
	{
		tagBahnDesignPt *pPt = &(pBD->Pts[id]);
		tabPlan->setDisabled(false);
		leId->setText(QString::number(id));
		leTime->setText(QString::number(pPt->time));
		leSpeed->setText(QString::number(pPt->Speed));
		leX->setText(QString::number(pPt->Vect.x));
		leY->setText(QString::number(pPt->Vect.y));
		leAlpha->setText(QString::number(pPt->Vect.alpha));
	}
}

/*----------------------  Save Data in PlanSheet  ----------------------------*/
void Form1::SaveBDPtData()
{
	if ( SelBDId >= 0 ) 
	{
		tagBahnDesignPt &pt = pBD->Pts[SelBDId];
		tabPlan->setDisabled(false);
		pt.time = leTime->text().toInt();
		pt.Speed = leSpeed->text().toInt();
		pt.Vect.x = leX->text().toDouble();
		pt.Vect.y = leY->text().toDouble();
		pt.Vect.alpha = leAlpha->text().toDouble();
	}
}

/*----------------------------------------------------------------------------*/
void Form1::SavePara()
{
	tagPara &para = m_Para;
	para.MinR = leMinR->text().toDouble();
	para.MaxR = leMaxR->text().toDouble();
	para.PlusA = lePlusA->text().toDouble();
	para.NegA = leNegA->text().toDouble();
	para.SpeedFact = leSpeedFact->text().toDouble();
	para.LenkFact = leLenkFact->text().toDouble();
	para.MaxSpeed = leMaxSpeed->text().toDouble();
	para.MinSpeedHold = leMinSpeedHold->text().toDouble();
}

/*----------------------------------------------------------------------------*/
void Form1::ShowPara()
{
	tagPara &para = m_Para;
	leMinR->setText(QString::number(para.MinR));
	leMaxR->setText(QString::number(para.MaxR));
	lePlusA->setText(QString::number(para.PlusA));
	leNegA->setText(QString::number(para.NegA));
	leSpeedFact->setText(QString::number(para.SpeedFact));
	leLenkFact->setText(QString::number(para.LenkFact));
	leMaxSpeed->setText(QString::number(para.MaxSpeed));
	leMinSpeedHold->setText(QString::number(para.MinSpeedHold));
}

/*----------------------------------------------------------------------------*/
void Form1::SetCurTool(BTool tTool)
{
	CurTool = tTool;
	Viewer->setCursor( BTCursor[tTool] );
	ShowCurStatus();
}

/*----------------------------------------------------------------------------*/
void Form1::fileExportClick()
{
	QString s = QFileDialog::getSaveFileName(
					"",
                    "",
                    this,
                    "save file dialog"
                    "Choose a filename to save under" );
	ofstream fo;
	fo.open(s);

	int Ct = pBD->CtlPtList.GetCount();

	for (int i=0;i<Ct;i++)
	{
		QString str = pBD->CtlPtList.GetCmdF(i);
		fo << str << endl;
	}
	
	fo.close();
}

/*----------------------------------------------------------------------------*/
void Form1::fileImportClick()
{
	QString s = QFileDialog::getOpenFileName(
                    "",
                    "",
                    this,
                    "open file dialog"
                    "Choose a file" );

    QFile file( s );
    if ( !file.open( IO_ReadOnly ) )
	{
		QMessageBox::critical( 0, "", "Can't Open file!");
        return;
	}
	
	QString ltxt;
	tagPos pos = {0, 0, {0, 0, 0}};
	
	if (pBD->Count>0)
	{
		pos.time = pBD->Pts[0].time;
		pos.vect = pBD->Pts[0].Vect;
	}

	pTrack->Reset(pos);

	while (file.readLine(ltxt, 100)>=0)
	{
		pTrack->AppendCmd(ltxt);
	}

    file.close();

	Viewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::mSavePlanClick()
{
	QString s = QFileDialog::getSaveFileName(
					"",
                    "",
                    this,
                    "save file dialog"
                    "Choose a filename to save under" );

	ofstream fo;
	fo.open(s);

    QDomDocument doc;
    QDomElement root = doc.createElement( "BahnPlan" );
    doc.appendChild( root );

	for (int i=0;i<pBD->Count;i++)
	{
		QString txt;

		QDomElement BDPt = doc.createElement( "BahnDesignPt" );
		root.appendChild( BDPt );
		BDPt.setAttribute("id", i);
		BDPt.setAttribute("time", pBD->Pts[i].time);
		BDPt.setAttribute("Speed", pBD->Pts[i].Speed);

		QDomElement Vect = doc.createElement( "Vect" );
		BDPt.appendChild( Vect );
		Vect.setAttribute("x", pBD->Pts[i].Vect.x);
		Vect.setAttribute("y", pBD->Pts[i].Vect.y);
		Vect.setAttribute("alpha", pBD->Pts[i].Vect.alpha);
	}

    QString DocTxt = doc.toString();
	fo << DocTxt;
	
	fo.close();
}

/*----------------------------------------------------------------------------*/
void Form1::mLoadPlanClick()
{
	QString s = QFileDialog::getOpenFileName(
                    "",
                    "",
                    this,
                    "open file dialog"
                    "Choose a file" );

	QDomDocument doc;
    QFile file( s );
    if ( !file.open( IO_ReadOnly ) )
	{
		QMessageBox::critical( 0, "", "Can't Open file!");
        return;
	}
    if ( !doc.setContent( &file, false) ) {
        file.close();
		QMessageBox::critical( 0, "", "Bad file!");
        return;
    }
    file.close();

    pBD->Count = 0;
    pBD->StkCt = 0;
	
	// parse
    QDomElement root = doc.documentElement();
    QDomNode n = root.firstChild();

    while( !n.isNull() ) 
	{
        QDomElement BDPt = n.toElement();
        if( !BDPt.isNull() ) 
		{
			tagBahnDesignPt *pN = pBD->add();
			pN->time = BDPt.attribute("time").toInt();
			pN->Speed = BDPt.attribute("Speed").toDouble();

			QDomNode m = n.firstChild();

			if ( !m.isNull())
			{
				QDomElement vect = m.toElement();
				if( !vect.isNull() ) 
				{
					pN->Vect.x = vect.attribute("x").toDouble();
					pN->Vect.y = vect.attribute("y").toDouble();
					pN->Vect.alpha = vect.attribute("alpha").toDouble();
				}
			}
        }
        n = n.nextSibling();
    }

	pBD->Calc();
	Viewer->repaint();
	SpeedViewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::mAddClick()
{
    SetCurTool(BT_Add);
}

/*----------------------------------------------------------------------------*/
void Form1::mDeleteClick()
{
    SetCurTool(BT_Delete);
}

/*----------------------------------------------------------------------------*/
void Form1::mMoveClick()
{
    SetCurTool(BT_Move);
}

/*----------------------------------------------------------------------------*/
void Form1::mClearClick()
{
	pBD->Count = 0;    
	pBD->Calc();
	Viewer->repaint();
	SpeedViewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::mZoomInClick()
{
	//matrix.scale(2,2);
	ct->Scale(2);
	Viewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::mZoomOutClick()
{
	//matrix.scale(0.5,0.5);
	ct->Scale(0.5);
	Viewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::mVMoveClick()
{
	SetCurTool(V_Move1);
}

/*----------------------------------------------------------------------------*/
void Form1::btOK_clicked()
{
	Viewer->drawBahnDesignPt(SelBDId, false);
	SaveBDPtData();
	Viewer->drawBahnDesignPt(SelBDId, true);

	pBD->Calc();
	Viewer->repaint();
	SpeedViewer->repaint();
}

/*----------------------------------------------------------------------------*/
void Form1::btOK_2_clicked()
{
	SavePara();

	pBD->Calc();
	Viewer->repaint();
	SpeedViewer->repaint();
}

/*----------------------------------------------------------------------------*/
Form1::Form1( QWidget* parent, const char* name, WFlags fl )
    : QMainWindow( parent, name, fl )
{
    (void)statusBar();
    if ( !name )
    {
		setName( "BahnPlaner" );
	}
	
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );
    Form1Layout = new QHBoxLayout( centralWidget(), 0, 2, "Form1Layout"); 

    frame4 = new QFrame( centralWidget(), "frame4" );
    frame4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, frame4->sizePolicy().hasHeightForWidth() ) );
    frame4->setFrameShape( QFrame::StyledPanel );
    frame4->setFrameShadow( QFrame::Raised );
    frame4Layout = new QVBoxLayout( frame4, 0, 2, "frame4Layout"); 

    Viewer = new ViewerWidget( frame4, "Viewer" );
    Viewer->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, Viewer->sizePolicy().hasHeightForWidth() ) );
    Viewer->setMinimumSize( QSize( 400, 390 ) );
    Viewer->setPaletteBackgroundColor( clrVBG );
    Viewer->pSBar=statusBar();
    Viewer->setMouseTracking(true);
    frame4Layout->addWidget( Viewer );

    SpeedViewer = new SpeedWidget( frame4, "SpeedViewer" );
    SpeedViewer->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, SpeedViewer->sizePolicy().hasHeightForWidth() ) );
    SpeedViewer->setMinimumSize( QSize( 0, 125 ) );
    SpeedViewer->setPaletteBackgroundColor( clrVBG );
    frame4Layout->addWidget( SpeedViewer );
    Form1Layout->addWidget( frame4 );

    tabWidget2 = new QTabWidget( centralWidget(), "tabWidget2" );
    tabWidget2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)7, 0, 0, tabWidget2->sizePolicy().hasHeightForWidth() ) );
    tabWidget2->setMinimumSize( QSize( 130, 0 ) );

    tabPlan = new QWidget( tabWidget2, "tabPlan" );

    lbId = new QLabel( tabPlan, "lbId" );
    lbId->setGeometry( QRect( 10, 20, 60, 20 ) );

    leId = new QLineEdit( tabPlan, "leId" );
    leId->setGeometry( QRect( 10, 40, 106, 21 ) );
    leId->setReadOnly(true);

    lbTime = new QLabel( tabPlan, "lbTime" );
    lbTime->setGeometry( QRect( 10, 70, 60, 20 ) );

    leTime = new QLineEdit( tabPlan, "leTime" );
    leTime->setGeometry( QRect( 10, 90, 106, 21 ) );

    lbX = new QLabel( tabPlan, "lbX" );
    lbX->setGeometry( QRect( 10, 120, 60, 20 ) );

    leX = new QLineEdit( tabPlan, "leX" );
    leX->setGeometry( QRect( 10, 140, 106, 21 ) );

    lbY = new QLabel( tabPlan, "textLabel1_3" );
    lbY->setGeometry( QRect( 10, 170, 60, 20 ) );

    leY = new QLineEdit( tabPlan, "leY" );
    leY->setGeometry( QRect( 10, 190, 106, 21 ) );

    lbAlpha = new QLabel( tabPlan, "textLabel1_4" );
    lbAlpha->setGeometry( QRect( 10, 220, 60, 20 ) );

    leAlpha = new QLineEdit( tabPlan, "leAlpha" );
    leAlpha->setGeometry( QRect( 10, 240, 106, 21 ) );

    lbSpeed = new QLabel( tabPlan, "textLabel1_5" );
    lbSpeed->setGeometry( QRect( 10, 270, 60, 20 ) );

    leSpeed = new QLineEdit( tabPlan, "leSpeed" );
    leSpeed->setGeometry( QRect( 10, 290, 106, 21 ) );

    btOK = new QPushButton( tabPlan, "btOK" );
    btOK->setGeometry( QRect( 40, 430, 50, 20 ) );

    tabWidget2->insertTab( tabPlan, QString("") );

    tabPara = new QWidget( tabWidget2, "tabPara" );

    lbMaxR = new QLabel( tabPara, "lbMaxR" );
    lbMaxR->setGeometry( QRect( 10, 70, 60, 20 ) );

    lbMinR = new QLabel( tabPara, "lbMinR" );
    lbMinR->setGeometry( QRect( 10, 20, 60, 20 ) );

    leMinR = new QLineEdit( tabPara, "leMinR" );
    leMinR->setGeometry( QRect( 10, 40, 106, 21 ) );

    leMaxR = new QLineEdit( tabPara, "leMaxR" );
    leMaxR->setGeometry( QRect( 10, 90, 106, 21 ) );

    lePlusA = new QLineEdit( tabPara, "lePlusA" );
    lePlusA->setGeometry( QRect( 10, 140, 106, 21 ) );

    lbPlusA = new QLabel( tabPara, "lbPlusA" );
    lbPlusA->setGeometry( QRect( 10, 120, 60, 20 ) );

    lbNegA = new QLabel( tabPara, "lbNegA" );
    lbNegA->setGeometry( QRect( 10, 170, 60, 20 ) );

    leNegA = new QLineEdit( tabPara, "leNegA" );
    leNegA->setGeometry( QRect( 10, 190, 106, 21 ) );

    lbMaxSpeed = new QLabel( tabPara, "lbMaxSpeed" );
    lbMaxSpeed->setGeometry( QRect( 10, 220, 60, 20 ) );

    leMaxSpeed = new QLineEdit( tabPara, "leMaxSpeed" );
    leMaxSpeed->setGeometry( QRect( 10, 240, 106, 21 ) );

    lbSpeedFact = new QLabel( tabPara, "lbSpeedFact" );
    lbSpeedFact->setGeometry( QRect( 10, 270, 60, 20 ) );

    leSpeedFact = new QLineEdit( tabPara, "leSpeedFact" );
    leSpeedFact->setGeometry( QRect( 10, 290, 106, 21 ) );

    lbLenkFact = new QLabel( tabPara, "lbLenkFact" );
    lbLenkFact->setGeometry( QRect( 10, 320, 60, 20 ) );

    leLenkFact = new QLineEdit( tabPara, "leLenkFact" );
    leLenkFact->setGeometry( QRect( 10, 340, 106, 21 ) );

    lbMinSpeedHold = new QLabel( tabPara, "lbMinSpeedHold" );
    lbMinSpeedHold->setGeometry( QRect( 10, 370, 60, 20 ) );


    leMinSpeedHold = new QLineEdit( tabPara, "leMinSpeedHold" );
    leMinSpeedHold->setGeometry( QRect( 10, 390, 106, 21 ) );

    btOK_2 = new QPushButton( tabPara, "btOK_2" );
    btOK_2->setGeometry( QRect( 40, 430, 50, 20 ) );

    tabWidget2->insertTab( tabPara, QString("") );
    Form1Layout->addWidget( tabWidget2 );

    // actions
    mAdd = new QAction( this, "mAdd" );
    mDelete = new QAction( this, "mDelete" );
    mMove = new QAction( this, "mMove" );
    mClear = new QAction( this, "mClear" );
	mSavePlan = new QAction( this, "mSavePlan" );
	mLoadPlan = new QAction( this, "mLoadPlan" );
    fileExportAction = new QAction( this, "fileExportAction" );
    fileImportAction = new QAction( this, "fileImportAction" );
    mVMove = new QAction( this, "mVMove" );
    mZoomIn = new QAction( this, "mZoomIn" );
    mZoomOut = new QAction( this, "mZoomOut" );

    // menubar
    Menubar = new QMenuBar( this, "Menubar" );

    mFile = new QPopupMenu( this );
	mSavePlan->addTo( mFile );
	mLoadPlan->addTo( mFile );
    fileExportAction->addTo( mFile );
    fileImportAction->addTo( mFile );
    Menubar->insertItem( QString(""), mFile, 1 );

    mBahn = new QPopupMenu( this );
    mAdd->addTo( mBahn );
    mDelete->addTo( mBahn );
    mMove->addTo( mBahn );
    mClear->addTo( mBahn );
    Menubar->insertItem( QString(""), mBahn, 2 );

    mView = new QPopupMenu( this );
    mVMove->addTo( mView );
    mZoomIn->addTo( mView );
    mZoomOut->addTo( mView );
    Menubar->insertItem( QString(""), mView, 3 );

    languageChange();
    resize( QSize(520, 498).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    // signals and slots connections
    connect( mSavePlan, SIGNAL( activated() ), this, SLOT( mSavePlanClick() ) );
    connect( mLoadPlan, SIGNAL( activated() ), this, SLOT( mLoadPlanClick() ) );
    connect( fileExportAction, SIGNAL( activated() ), this, SLOT( fileExportClick() ) );
    connect( fileImportAction, SIGNAL( activated() ), this, SLOT( fileImportClick() ) );
    connect( mAdd, SIGNAL( activated() ), this, SLOT( mAddClick() ) );
    connect( mDelete, SIGNAL( activated() ), this, SLOT( mDeleteClick() ) );
    connect( mMove, SIGNAL( activated() ), this, SLOT( mMoveClick() ) );
    connect( mClear, SIGNAL( activated() ), this, SLOT( mClearClick() ) );
    connect( btOK, SIGNAL( clicked() ), this, SLOT( btOK_clicked() ) );
    connect( mVMove, SIGNAL( activated() ), this, SLOT( mVMoveClick() ) );
    connect( mZoomIn, SIGNAL( activated() ), this, SLOT( mZoomInClick() ) );
    connect( mZoomOut, SIGNAL( activated() ), this, SLOT( mZoomOutClick() ) );
    //connect( Viewer, SIGNAL( mouseMoveEvent(QMouseEvent*) ), this, SLOT( Viewer_mouseMoveEvent(QMouseEvent*) ) );
    connect( btOK_2, SIGNAL( clicked() ), this, SLOT( btOK_2_clicked() ) );

	// Init
	ShowPara();
}

/*----------------------------------------------------------------------------*/
Form1::~Form1()
{
    
}

/*----------------------  Set  all texts of componets  -----------------------*/
void Form1::languageChange()
{
    setCaption( tr( "Bahnplaner" ) );
    lbId->setText( tr( "id:" ) );
    lbTime->setText( tr( "time:" ) );
    lbX->setText( tr( "x:" ) );
    lbY->setText( tr( "y:" ) );
    lbAlpha->setText( tr( "alpha:" ) );
    lbSpeed->setText( tr( "speed:" ) );
    btOK->setText( tr( "OK" ) );
    tabWidget2->changeTab( tabPlan, tr( "Plan" ) );
    lbMinR->setText( tr( "MinR:" ) );
    lbMaxR->setText( tr( "MaxR:" ) );
    lbPlusA->setText( tr( "PlusA:" ) );
    lbNegA->setText( tr( "NegA:" ) );
    lbMaxSpeed->setText( tr( "MaxSpeed:" ) );
    lbSpeedFact->setText( tr( "SpeedFact:" ) );
    lbLenkFact->setText( tr( "LenkFact:" ) );
    lbMinSpeedHold->setText( tr( "MinSpeedHold:" ) );
    btOK_2->setText( tr( "OK" ) );
    tabWidget2->changeTab( tabPara, tr( "Parameter" ) );
    mAdd->setText( tr( "Add" ) );
    mDelete->setText( tr( "Delete" ) );
    mMove->setText( tr( "Move" ) );
    mClear->setText( tr( "Clear" ) );
	mSavePlan->setText( tr( "SavePlan" ) );
	mLoadPlan->setText( tr( "LoadPlan" ) );
    fileExportAction->setText( tr( "Export" ) );
    fileImportAction->setText( tr( "Import" ) );
    mVMove->setText( tr( "Move" ) );
    mZoomIn->setText( tr( "Zoom In" ) );
    mZoomOut->setText( tr( "Zoom Out" ) );
    Menubar->findItem( 1 )->setText( tr( "File" ) );
    Menubar->findItem( 2 )->setText( tr( "Bahn" ) );
    Menubar->findItem( 3 )->setText( tr( "View" ) );
}


/***********************  main function  **************************************/
int main( int argc, char ** argv )
{
	QApplication a( argc, argv );
    pBD = new BahnDesign();
    pTrack = new Track();
    pForm1 = new Form1();
    a.setMainWidget(pForm1);
    a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
    pForm1->show();
    int rtn = a.exec();
    
    if (m_Log.is_open())
    {
      m_Log.close();
    }
    
    delete pForm1;
    delete pBD;
    delete pTrack;
    return rtn;
}
